﻿using AutoMapper;
using Countries.NET;
using Microsoft.AspNetCore.Identity;
using MTWorkHR.Application.Models;
using MTWorkHR.Core.Entities;
using MTWorkHR.Core.Entities.User;
using MTWorkHR.Core.Global;
using MTWorkHR.Infrastructure.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace MTWorkHR.Application.Mapper
{
    public class MappingProfile : Profile
    {
        public MappingProfile() {

            //identity user

            CreateMap<UserDto, ApplicationUser>()
                .ForMember(m => m.UserRoles, op => op.Ignore())
                .ForMember(m => m.Id, op => op.Ignore());

            CreateMap<ApplicationUser, UserDto>().ForMember(m => m.Password, op => op.Ignore());
            CreateMap<ApplicationUser, UserAllDto>()
                .ForMember(s => s.QualificationName, o => o.MapFrom(s => s.Qualification ==null ? "" : GlobalInfo.lang == "ar" ? s.Qualification.NameAr : s.Qualification.NameEn))
                .ForMember(s => s.JobTitleName, o => o.MapFrom(s => s.JobTitle ==null ? "" : GlobalInfo.lang == "ar" ? s.JobTitle.NameAr : s.JobTitle.NameEn))
                .ForMember(s => s.IndustryName, o => o.MapFrom(s => s.Industry ==null ? "" : GlobalInfo.lang == "ar" ? s.Industry.NameAr : s.Industry.NameEn))
                .ForMember(s => s.CountryName, o => o.MapFrom(s => s.Country ==null ? "" : GlobalInfo.lang == "ar" ? s.Country.NameAr : s.Country.NameEn))
                .ForMember(s => s.UniversityName, o => o.MapFrom(s => s.University == null ? "" : GlobalInfo.lang == "ar" ? s.University.NameAr : s.University.NameEn)
                );
            
            CreateMap<UserUpdateDto, ApplicationUser>()
                .ForMember(m => m.UserRoles, op => op.Ignore())
               // .ForMember(m => m.UserBranchs, op => op.Ignore())
               // .ForMember(m => m.UserCycleApprovals, op => op.Ignore())
                //.ForMember(m => m.Hierarchies, op => op.Ignore())
                .ForMember(m => m.Id, op => op.Ignore());

            CreateMap<ApplicationUser, UserUpdateDto>();

            CreateMap<AttachmentDto, UserAttachment>().ReverseMap();
            CreateMap<UserAddress, UserAddressDto>().ReverseMap();

            //identity userRoles
            CreateMap<IdentityUserRole<string>, UserRoleDto>().ReverseMap();

            CreateMap<ApplicationRole, UserRoleDto>()
                .ForMember(m => m.RoleId, op => op.MapFrom(mp => mp.Id))
                .ForMember(m => m.RoleName, op => op.MapFrom(mp => mp.Name))
                .ReverseMap();

            //identity role
            CreateMap<RoleDto, ApplicationRole>()
                .ForMember(m => m.Id, op => op.Ignore())
                .ForMember(m => m.RolePermissions, op => op.Ignore())
                ;
            CreateMap<ApplicationRole, RoleDto>();

            CreateMap<RolePermissionDto, RolePermission>().ReverseMap();

            CreateMap<ProjectDto, Project>().ForMember(d => d.CreateDate, o => o.Ignore()).ReverseMap();
            CreateMap<UserTaskDto, UserTask>().ForMember(d => d.CreateDate, o => o.Ignore()).ReverseMap();
            CreateMap<UserTaskAttachment, AttachmentDto>().ReverseMap();
            CreateMap<UserTaskHistoryDto, UserTaskHistory>().ForMember(d => d.CreateDate, o => o.Ignore()).ReverseMap();
            CreateMap<Team, TeamDto>().ReverseMap().ForMember(d => d.CreateDate, o => o.Ignore());
            CreateMap<TeamUserDto, TeamUser>().ForMember(d => d.CreateDate, o => o.Ignore()).ReverseMap();
            CreateMap<Meeting, MeetingDto>().ReverseMap()
                .ForMember(d => d.CreateDate, o => o.Ignore());
            CreateMap<MeetingUser, MeetingUserDto>().ReverseMap().ForMember(d => d.CreateDate, o => o.Ignore());

            CreateMap<Attendance, AttendanceDto>().ForMember(d=> d.WeekDay , o=> o.MapFrom(s=> s.AttendanceDate.DayOfWeek)).ReverseMap()
                .ForMember(d => d.CreateDate, o => o.Ignore());

            CreateMap<OrderAllocation, OrderAllocationDto>().ReverseMap().ForMember(d => d.CreateDate, o => o.Ignore());
            CreateMap<OrderRequest, OrderRequestDto>().ReverseMap().ForMember(d => d.CreateDate, o => o.Ignore());
            CreateMap<OrderType, OrderTypeDto>().ReverseMap();
            CreateMap<LeaveType, LeaveTypeDto>().ReverseMap();
            CreateMap<Company, CompanyDto>().ReverseMap().ForMember(d => d.CreateDate, o => o.Ignore());
            CreateMap<CountryLookup, CountryDto>().ReverseMap();
            CreateMap<Industry, IndustryDto>().ReverseMap();
            CreateMap<JobTitle, JobTitleDto>().ReverseMap();


        }
    }
}
